/**
* Copyright 2007 - 2010 Skyway Software, Inc.
*/
package org.skyway.spring.util.webservice.cxf;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.cxf.binding.soap.saaj.SAAJInInterceptor;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.AbstractPhaseInterceptor;
import org.apache.cxf.phase.Phase;
import org.apache.cxf.ws.security.wss4j.WSS4JInInterceptor;
import org.apache.ws.security.handler.WSHandlerConstants;

/**
 * Authenticates incoming web service calls.  If security is enabled, the AuthenticationManager
 * will be injected by spring as will the list of secured service end points.
 * 
 * @author jperkins
 *
 */
public class WSSecurityInInterceptor extends AbstractPhaseInterceptor<Message> {
	private SpringSecurityCallbackHandler callbackHandler;
	private List<String> securedServiceEndPointNames;
	
	public WSSecurityInInterceptor() {
		super(Phase.PRE_PROTOCOL);
	}

	public WSSecurityInInterceptor(String s) {
		super(Phase.PRE_PROTOCOL);
	}

	@SuppressWarnings("unchecked")
	public void handleMessage(Message message) throws Fault {
		/*
		 * Only check if we need to authenticate if we are not the requestor. If we are the
		 * requestor than we sent the request and it is now coming back in.
		 */
		if (!Boolean.TRUE.equals(message.get(Message.REQUESTOR_ROLE))) {
			if (isSecurityEnabled(message)) {
				Map props = new HashMap();
				props.put(WSHandlerConstants.ACTION, WSHandlerConstants.USERNAME_TOKEN);
				props.put(WSHandlerConstants.PW_CALLBACK_REF, callbackHandler);

				WSS4JInInterceptor wss4jInHandler = new WSS4JInInterceptor(props);

				message.getInterceptorChain().add(wss4jInHandler);
				message.getInterceptorChain().add(new SAAJInInterceptor());
			}
		}
	}

	private boolean isSecurityEnabled(Message message) {
		// If authenticationProvider hasn't been set, it means that security is not enabled
		if (callbackHandler == null) {
			return false;
		}
		
		// The path info has the form:
		// /<project name>/<service endpoint non-qualified interface name>
		// i.e.,
		// /ServerSecurityWS/IHelloWorldServiceServiceEndpoint
		String pathInfo = (String)message.get("org.apache.cxf.message.Message.PATH_INFO"); //$NON-NLS-1$
		
		String serviceEndpoint = StringUtils.substringAfterLast(pathInfo, "/"); //$NON-NLS-1$
		
		return getSecuredServiceEndPointNames().contains(serviceEndpoint);
	}
	
	public void setCallbackHandler(SpringSecurityCallbackHandler callbackHandler) {
		this.callbackHandler = callbackHandler;
	}

	public List<String> getSecuredServiceEndPointNames() {
		return securedServiceEndPointNames;
	}

	public void setSecuredServiceEndPointNames(List<String> securedServiceEndPointNames) {
		this.securedServiceEndPointNames = securedServiceEndPointNames;
	}
}
